"""
Module realize lists mixin
"""

from typing import Awaitable, List, Optional, Union

from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.lunavl.mixins.base import MixinStub


class ListsMixin(MixinStub):
    """Lists Mixin to work with lists."""

    def createList(self, userData: Optional[str] = None, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create new list

        Args:
            userData: user data associated with the list (<= 128 characters)

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: information about created list

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.createList("strange_persons")
        >>> print(response.statusCode)
        201
        >>> print(response.json)
        {
          "list_id":"366fc641-f502-47c4-8bcc-b6aba4a3122b",
          "url":"\\/1\\/lists\\/366fc641-f502-47c4-8bcc-b6aba4a3122b"
        }

        """
        payload = RequestPayload.buildJson(body={"user_data": userData}) if userData is not None else None
        return self.makeRequest(f"{self.baseUri}/lists", "POST", body=payload, **kwargs)

    def getLists(
        self,
        createTimeLt: Optional[str] = None,
        createTimeGte: Optional[str] = None,
        lastUpdateTimeLt: Optional[str] = None,
        lastUpdateTimeGte: Optional[str] = None,
        userData: Optional[str] = None,
        page: int = 1,
        pageSize: int = 10,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get lists by filters

        Args:
            userData: Find all objects with user_data which is "like" this parameter
            createTimeLt: upper bound of list create time
            createTimeGte: lower bound of list create time
            lastUpdateTimeLt: upper bound of list last update time
            lastUpdateTimeGte: lower bound of list last update time
            page: The number of page (>= 1)
            pageSize: The numbers of items on page [ 1 .. 100 ]

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: lists that were found

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.getLists(userData="strange")
        >>> print(response.statusCode)
        200
        >>> print(response.json)
        [
          {
            "list_id": "366fc641-f502-47c4-8bcc-b6aba4a3122b",
            "user_data": "strange_persons",
            "create_time": "2018-09-24T12:44:31.111600+03:00",
            "last_update_time": "2018-09-24T12:44:31.111615+03:00"
          },
          {
            "list_id": "34e07ba0-2e0e-4cab-8384-d9020398f442",
            "user_data": "strange_persons",
            "create_time": "2018-09-24T12:44:01.781151+03:00",
            "last_update_time": "2018-09-24T12:44:01.781174+03:00"
          }
        ]

        """
        queries = {
            "page": page,
            "page_size": pageSize,
            "user_data": userData,
            "create_time__lt": createTimeLt,
            "create_time__gte": createTimeGte,
            "last_update_time__lt": lastUpdateTimeLt,
            "last_update_time__gte": lastUpdateTimeGte,
        }
        return self.makeRequest(f"{self.baseUri}/lists", "GET", queryParams=queries, **kwargs)

    def getListsCount(
        self,
        listIds: Optional[List[str]] = None,
        createTimeLt: Optional[str] = None,
        createTimeGte: Optional[str] = None,
        lastUpdateTimeLt: Optional[str] = None,
        lastUpdateTimeGte: Optional[str] = None,
        userData: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get lists count by filters

        Args:
            userData: Find all objects with user_data which is "like" this parameter
            listIds: list ids
            createTimeLt: upper bound of list create time
            createTimeGte: lower bound of list create time
            lastUpdateTimeLt: upper bound of list last update time
            lastUpdateTimeGte: lower bound of list last update time

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: count of lists that were found

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.getListsCount(userData="strange")
        >>> print(response.statusCode)
        200
        >>> print(response.json)
        2

        """
        queries = {
            "user_data": userData,
            "list_ids": listIds,
            "create_time__lt": createTimeLt,
            "create_time__gte": createTimeGte,
            "last_update_time__lt": lastUpdateTimeLt,
            "last_update_time__gte": lastUpdateTimeGte,
        }
        return self.makeRequest(f"{self.baseUri}/lists/count", "GET", queryParams=queries, **kwargs)

    def getList(self, listId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get list by id

        Args:
            listId: list id (uuid4)

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: info about list

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.getList("366fc641-f502-47c4-8bcc-b6aba4a3122b")
        >>> print(response.statusCode)
        200
        >>> print(response.json)
        {
          "list_id": "366fc641-f502-47c4-8bcc-b6aba4a3122b",
          "user_data": "strange_persons",
          "create_time": "2018-09-24T12:44:31.111600+03:00",
          "last_update_time": "2018-09-24T12:44:31.111615+03:00"
        }

        """
        return self.makeRequest(f"{self.baseUri}/lists/{listId}", "GET", **kwargs)

    def checkList(self, listId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check a list with id=list_id existence

        Args:
            listId: list id (uuid4)

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: nothing

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.getList("366fc641-f502-47c4-8bcc-b6aba4a3122b")
        >>> print(response.statusCode)
        200
        >>> print(response.text)

        """
        return self.makeRequest(f"{self.baseUri}/lists/{listId}", "HEAD", **kwargs)

    def updateList(self, listId: str, userData: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Update the list user_data

        Args:
            listId: list id (uuid4)
            userData: user data associated with the list (<= 128 characters)

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: nothing

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.updateList("366fc641-f502-47c4-8bcc-b6aba4a3122b", userData="test_data12_updated")
        >>> print(response.statusCode)
        204
        >>> print(response.text)

        """
        payload = RequestPayload.buildJson(body={"user_data": userData})
        return self.makeRequest(f"{self.baseUri}/lists/{listId}", "PATCH", body=payload, **kwargs)

    def deleteList(self, listId: str, withFaces: int = 0, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Remove list by id

        Args:
            listId: list id (uuid4)
            withFaces: remove lists and all the faces contained in the lists

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: nothing

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.deleteList("3a8decfa-e42d-4bec-b758-5077b2893758")
        >>> print(response.statusCode)
        204
        >>> print(response.text)

        """
        queries = {"with_faces": withFaces}
        return self.makeRequest(f"{self.baseUri}/lists/{listId}", "DELETE", queryParams=queries, **kwargs)

    def linkFacesToList(
        self, listId: str, action: str, faceIds: list, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        attach/detach faces to the list

        Args:
            listId: list id (uuid4)
            action: expected action with list and faces (Enum:"attach" "detach")
            faceIds: faces for attach or detach to/from list

        Raises:
            ValueError: if action parameter not in ("attach", "detach")

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: nothing

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.linkFacesToList(listId="366fc641-f502-47c4-8bcc-b6aba4a3122b", action='attach',
        >>>                                     faceIds=['3da33513-1274-469f-84fd-b66d1c459a54',
        >>>                                              '51fcd44c-63f8-47ee-894d-31fbbbe1e8b7'])
        >>> print(response.statusCode)
        204
        >>> print(response.text)
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> response = client.linkFacesToList(listId="366fc641-f502-47c4-8bcc-b6aba4a3122b", action='detach',
        >>>                                     faceIds=['3da33513-1274-469f-84fd-b66d1c459a54',
        >>>                                              '51fcd44c-63f8-47ee-894d-31fbbbe1e8b7'])
        >>> print(response.statusCode)
        204
        >>> print(response.text)

        """
        if action not in ("attach", "detach"):
            raise ValueError("Attach parameter should be equal to 'attach' or 'detach'")
        payload = RequestPayload.buildJson(body={"action": action, "face_ids": faceIds})
        return self.makeRequest(f"{self.baseUri}/lists/{listId}/faces", "PATCH", body=payload, **kwargs)
