"""
Module realize matcher mixin
"""

from typing import Awaitable, List, Optional, Union

from luna3.backport4.matcher_objects import Candidates, Reference
from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.lunavl.mixins.base import MixinStub


class MatcherMixin(MixinStub):
    """Matcher Mixin to work with matcher."""

    def match(
        self,
        candidates: Candidates,
        references: List[Reference],
        limit: int = 3,
        targets: Optional[List[str]] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Matcher API allows to submit tasks to a service that searches for faces similar to a given reference(s)
        by matching them. An attribute descriptor should be extracted for reference(s) and candidate(s).
        A source for references are events, attributes and faces. A source for candidates are faces and lists.


        Args:
            candidates: Candidates object with info about candidates
            references: list of Reference objects. Each object has entity id and it type
                        (possible type variants: events, attributes, faces)
            limit: limit of returned results
            targets: list of candidate face fields for the match result enrichment

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: results of matching (failed and succeeded matches)

        >>> from luna3.common.http_objs import BinaryImage
        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> # greatFootballPlayersList - list with 3 faces of football players
        >>> # (Cristiano Ronaldo, Kylian Mbappe, Lionel Messi)
        >>> apiClient = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> greatFootballPlayersList = "60de7a7e-15f5-42f7-ae45-024c2184d0cd"
        >>> imagePath = "./data/Lionel_Messi01.jpg"
        >>> resp = apiClient.detectFaces(images=[BinaryImage(imagePath)])
        >>> sampleId = resp.json["succeeded_images"][0]["samples"][0]["sample_id"]
        >>> attributeId = apiClient.extractAttrFromSample(sampleIds=[sampleId]).json[0]["attribute_id"]
        >>> faceId = apiClient.createFace(externalId="possible_face_of_Lionel_Messi",
        >>>                               attributeId=attributeId).json["face_id"]
        >>> # Match attribute with lists:
        >>> candidates = Candidates("lists", [greatFootballPlayersList])
        >>> reference = Reference("attribute", attributeId)
        >>> resp = apiClient.match(
        >>>     candidates=candidates,
        >>> references=[reference]
        >>> )
        >>> print(f"'Searched attribute id: {attributeId}'")
        'Searched attribute id: 32c42f34-52aa-46d2-8be3-25f312d5aa0a'
        >>> print(resp.statusCode)
        201
        >>> print(resp.json)
        {
          "failed_matches": [],
          "succeeded_matches": [
            {
              "reference_id": "32c42f34-52aa-46d2-8be3-25f312d5aa0a",
              "matches": [
                {
                  "similarity": 0.9774027467,
                  "face": {
                    "face_id": "433a7572-54ac-4476-8d9f-4d57e10ff434",
                    "attribute_id": "e4f4d0bb-5a4b-4b6e-9e5a-999e81b3593a",
                    "user_data": "",
                    "create_time": "2018-10-09T18:51:06.351306+03:00",
                    "event_id": None,
                    "external_id": "Lionel Messi",
                    "avatar": None,
                    "lists": []
                  },
                  "list_id": "60de7a7e-15f5-42f7-ae45-024c2184d0cd"
                },
                {
                  "similarity": 0.0301167127,
                  "face": {
                    "face_id": "f370f7c2-ff10-46ea-9cbb-d1f072007bd4",
                    "attribute_id": "fd74143f-46d1-43fa-872b-1e6d48a6db70",
                    "user_data": "",
                    "create_time": "2018-10-09T18:51:05.744492+03:00",
                    "event_id": None,
                    "external_id": "Kylian Mbappe",
                    "avatar": None,
                    "lists": []
                  },
                  "list_id": "60de7a7e-15f5-42f7-ae45-024c2184d0cd"
                },
                {
                  "similarity": 0.0217440594,
                  "face": {
                    "face_id": "677dc144-8da9-4d68-ac39-6e3d271f7ccb",
                    "attribute_id": "132e7aa3-f336-49f5-9c64-0e4af8179bdf",
                    "user_data": "",
                    "create_time": "2018-10-09T18:51:05.282519+03:00",
                    "event_id": None,
                    "external_id": "Cristiano Ronaldo",
                    "avatar": None,
                    "lists": []
                  },
                  "list_id": "60de7a7e-15f5-42f7-ae45-024c2184d0cd"
                }
              ]
            }
          ]
        }
        >>> # Match face with lists:
        >>> candidates = Candidates("lists", [greatFootballPlayersList])
        >>> reference = Reference("face", faceId)
        >>> resp = apiClient.match(candidates=candidates, references=[reference])
        >>> print(f"'Searched face id: {faceId}'")
        'Searched face id: 05cb9707-7de0-426d-acc4-d666a0a56661'
        >>> print(resp.statusCode)
        201
        >>> print(resp.json)
        {
          "failed_matches": [],
          "succeeded_matches": [
            {
              "reference_id": "05cb9707-7de0-426d-acc4-d666a0a56661",
              "matches": [
                {
                  "similarity": 0.9774027467,
                  "face": {
                    "face_id": "433a7572-54ac-4476-8d9f-4d57e10ff434",
                    "attribute_id": "e4f4d0bb-5a4b-4b6e-9e5a-999e81b3593a",
                    "user_data": "",
                    "create_time": "2018-10-09T18:51:06.351306+03:00",
                    "event_id": None,
                    "external_id": "Lionel Messi",
                    "avatar": None,
                    "lists": []
                  },
                  "list_id": "60de7a7e-15f5-42f7-ae45-024c2184d0cd"
                },
                {
                  "similarity": 0.0301167127,
                  "face": {
                    "face_id": "f370f7c2-ff10-46ea-9cbb-d1f072007bd4",
                    "attribute_id": "fd74143f-46d1-43fa-872b-1e6d48a6db70",
                    "user_data": "",
                    "create_time": "2018-10-09T18:51:05.744492+03:00",
                    "event_id": None,
                    "external_id": "Kylian Mbappe",
                    "avatar": None,
                    "lists": []
                  },
                  "list_id": "60de7a7e-15f5-42f7-ae45-024c2184d0cd"
                },
                {
                  "similarity": 0.0217440594,
                  "face": {
                    "face_id": "677dc144-8da9-4d68-ac39-6e3d271f7ccb",
                    "attribute_id": "132e7aa3-f336-49f5-9c64-0e4af8179bdf",
                    "user_data": "",
                    "create_time": "2018-10-09T18:51:05.282519+03:00",
                    "event_id": None,
                    "external_id": "Cristiano Ronaldo",
                    "avatar": None,
                    "lists": []
                  },
                  "list_id": "60de7a7e-15f5-42f7-ae45-024c2184d0cd"
                }
              ]
            }
          ]
        }
        >>> # Match face with faces:
        >>> greatFootballPlayers = ["8fc97603-47a7-4d39-b7f6-560c9051fe68", "d14daa86-ea2b-4008-b81f-6d89e09f2f31",
        >>>                         "3cf38ffd-423c-431e-9017-7d6386f46914"]
        >>> apiClient = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> imagePath = "/home/domain_user/data/Cristiano_Ronaldo02.jpg"
        >>> resp = apiClient.detectFaces(images=[BinaryImage(imagePath)])
        >>> sampleId = resp.json["succeeded_images"][0]["samples"][0]["sample_id"]
        >>> attributeId = apiClient.extractAttrFromSample(sampleIds=[sampleId]).json[0]["attribute_id"]
        >>> faceId = apiClient.createFace(externalId="possible_face_of_Lionel_Messi",
        >>>                               attributeId=attributeId).json["face_id"]
        >>>
        >>> candidates = Candidates("faces", greatFootballPlayers)
        >>> reference = Reference("face", faceId)
        >>> resp = apiClient.match(
        >>>     candidates=candidates,
        >>>     references=[reference]
        >>> )
        >>> print(f"'Searched face id: {faceId}'")
        'Searched face id: 14b6f8bc-a6f5-432a-9964-c932d5616a27'
        >>> print(resp.statusCode)
        201
        >>> print(resp.json)
        {
          "failed_matches": [],
          "succeeded_matches": [
            {
              "reference_id": "14b6f8bc-a6f5-432a-9964-c932d5616a27",
              "matches": [
                {
                  "similarity": 0.9908255935,
                  "face": {
                    "face_id": "3cf38ffd-423c-431e-9017-7d6386f46914",
                    "attribute_id": "0da71d72-dc54-4d77-8aa0-83ea0a2db57b",
                    "user_data": "",
                    "create_time": "2018-10-10T16:17:40.701488+03:00",
                    "event_id": None,
                    "external_id": "Cristiano Ronaldo",
                    "avatar": None,
                    "lists": []
                  }
                },
                {
                  "similarity": 0.0532866903,
                  "face": {
                    "face_id": "8fc97603-47a7-4d39-b7f6-560c9051fe68",
                    "attribute_id": "be7aa933-dcfd-4230-8dcc-74a59cd6ba98",
                    "user_data": "",
                    "create_time": "2018-10-10T16:17:41.623753+03:00",
                    "event_id": None,
                    "external_id": "Lionel Messi",
                    "avatar": None,
                    "lists": []
                  }
                },
                {
                  "similarity": 0.0075068995,
                  "face": {
                    "face_id": "d14daa86-ea2b-4008-b81f-6d89e09f2f31",
                    "attribute_id": "da6be2cc-4ce7-45b0-a213-c303681691a0",
                    "user_data": "",
                    "create_time": "2018-10-10T16:17:41.153761+03:00",
                    "event_id": None,
                    "external_id": "Kylian Mbappe",
                    "avatar": None,
                    "lists": []
                  }
                }
              ]
            }
          ]
        }
        >>> greatFootballPlayers = ["8fc97603-47a7-4d39-b7f6-560c9051fe68", "d14daa86-ea2b-4008-b81f-6d89e09f2f31",
        >>>                         "3cf38ffd-423c-431e-9017-7d6386f46914"]
        >>> apiClient = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> attributeId = "11111111-4111-4111-b111-111111111111"  # Attribute not exists!
        >>> candidates = Candidates("faces", greatFootballPlayers)
        >>> reference = Reference("attribute", attributeId)
        >>> resp = apiClient.match(candidates=candidates, references=[reference])
        >>> print(resp.statusCode)
        201
        >>> print(resp.json)
        {'failed_matches': [{
            'reference_id': '11111111-4111-4111-b111-111111111111',
            'error': {
                'desc': 'Failed to get DB record',
                'detail': 'luna_persistent/biometry/11111111-4111-4111-b111-111111111111/descriptor:
                    Error(2) AEROSPIKE_ERR_RECORD_NOT_FOUND',
                'error_code': 1001}}],
        'succeeded_matches': []}

        """
        payload = RequestPayload.buildJson(
            body={"candidates": candidates.asDict(), "references": [ref.asDict() for ref in references]}
        )
        query = {"limit": limit, "targets": targets}
        return self.makeRequest(f"{self.baseUri}/matcher", "POST", queryParams=query, body=payload, **kwargs)
