"""
Module realize tasks mixin
"""

from typing import Awaitable, List, Optional, Union

from luna3.backport4.http_objs import EventFilters, FaceFilters
from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.lunavl.mixins.base import MixinStub


class TasksMixin(MixinStub):
    """Tasks Mixin to work with tasks."""

    def getTasks(
        self,
        taskIds: Optional[List[int]] = None,
        createTimeLt: Optional[str] = None,
        createTimeGte: Optional[str] = None,
        endTimeLt: Optional[str] = None,
        endTimeGte: Optional[str] = None,
        taskType: Optional[int] = None,
        taskStatus: Optional[int] = None,
        page: Optional[int] = None,
        pageSize: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get tasks with filters.

        Args:
            taskIds: task ids
            createTimeLt: task create time upper excluding bound
            createTimeGte: task create time lower including bound
            endTimeLt: task end time upper excluding bound
            endTimeGte: task end time lower including bound
            taskType: task type filter
            taskStatus: task status filter
            page: page
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> reply = client.getTasks()
        >>> print(reply.statusCode)
        200
        >>> print(reply.text)
        {
            "tasks":[{
                "task_id":14980,
                "create_time":"2019-04-29T10:53:42.291555+03:00",
                "end_time":"2019-04-29T10:53:42.469748+03:00",
                "last_update_time":"2019-04-29T10:53:42.448865+03:00",
                "task_type":2,
                "task_status":5,
                "result_id":"40561cd9-f044-4e87-a138-c4fb9a107d6e",
                "count_task_parts_done":1,
                "count_task_parts_all":1,
                "content":{
                    "faces":{
                        "list_id":"db2bef54-e629-40a9-9ae7-4807f913688a"
                    }},
                "description":""
            }]
        }

        """
        query = {
            "task_ids": taskIds,
            "create_time__lt": createTimeLt,
            "create_time__gte": createTimeGte,
            "end_time__lt": endTimeLt,
            "end_time__gte": endTimeGte,
            "task_type": taskType,
            "task_status": taskStatus,
            "page": page,
            "page_size": pageSize,
        }
        return self.makeRequest(f"{self.baseUri}/tasks", "GET", queryParams=query, **kwargs)

    def getTasksCount(
        self,
        taskIds: Optional[List[int]] = None,
        createTimeLt: Optional[str] = None,
        createTimeGte: Optional[str] = None,
        endTimeLt: Optional[str] = None,
        endTimeGte: Optional[str] = None,
        taskType: Optional[int] = None,
        taskStatus: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get tasks count with filters.

        Args:
            taskIds: task ids
            createTimeLt: task create time upper excluding bound
            createTimeGte: task create time lower including bound
            endTimeLt: task end time upper excluding bound
            endTimeGte: task end time lower including bound
            taskType: task type filter
            taskStatus: task status filter

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> reply = client.getTasksCount()
        >>> print(reply.statusCode)
        200
        >>> print(reply.text)
        {"count":12}

        """
        query = {
            "task_ids": taskIds,
            "create_time__lt": createTimeLt,
            "create_time__gte": createTimeGte,
            "end_time__lt": endTimeLt,
            "end_time__gte": endTimeGte,
            "task_type": taskType,
            "task_status": taskStatus,
        }
        return self.makeRequest(f"{self.baseUri}/tasks/count", "GET", queryParams=query, **kwargs)

    def getTask(self, taskId: int, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get task by id.

        Args:
            taskId: task id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> reply = client.getTask(14980)
        >>> print(reply.statusCode)
        200
        >>> print(reply.text)
        {
            "task_id":14980,
            "create_time":"2019-04-29T10:53:42.291555+03:00",
            "end_time":"2019-04-29T10:53:42.469748+03:00",
            "last_update_time":"2019-04-29T10:53:42.448865+03:00",
            "task_type":2,
            "task_status":5,
            "result_id":"40561cd9-f044-4e87-a138-c4fb9a107d6e",
            "count_task_parts_done":1,
            "count_task_parts_all":1,
            "content":{
                "faces":{
                    "list_id":"db2bef54-e629-40a9-9ae7-4807f913688a"
                    }
                },
            "description":""
        }

        """
        return self.makeRequest(f"{self.baseUri}/tasks/{taskId}", "GET", **kwargs)

    def getTaskErrors(
        self, taskId: int, page: Optional[int] = None, pageSize: Optional[int] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get task errors by task id.

        Args:
            taskId: task id
            page: page
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> reply = client.getTaskErrors(14980)
        >>> print(reply.statusCode)
        200
        >>> print(reply.text)
        {
            "errors":[{
                "error_id":5808750,
                "task_id":14980,
                "subtask_id":69825,
                "error_code":28010,
                "description":"Objects not found",
                "detail":"Objects for clustering not found (empty set)",
                "additional_info":null,
                "error_time":"2019-04-29T10:53:42.417396+03:00"
            }]
        }

        """
        query = {"page": page, "page_size": pageSize}
        return self.makeRequest(f"{self.baseUri}/tasks/{taskId}/errors", "GET", queryParams=query, **kwargs)

    def getErrors(
        self,
        errorIds: Optional[List[int]] = None,
        taskIds: Optional[List[int]] = None,
        errorTimeLt: Optional[str] = None,
        errorTimeGte: Optional[str] = None,
        taskType: Optional[int] = None,
        errorCode: Optional[int] = None,
        page: Optional[int] = None,
        pageSize: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get errors with filters.

        Args:
            errorIds: error ids
            taskIds: task ids
            errorTimeLt: error time upper excluding bound
            errorTimeGte: error time lower including bound
            taskType: task type filter
            errorCode: task status filter
            page: page
            pageSize: page size

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> reply = client.getErrors()
        >>> print(reply.statusCode)
        200
        >>> print(reply.text)
        {
            "errors":[{
                "error_id":5808750,
                "task_id":14980,
                "subtask_id":69825,
                "error_code":28010,
                "description":"Objects not found",
                "detail":"Objects for clustering not found (empty set)",
                "additional_info":null,
                "error_time":"2019-04-29T10:53:42.417396+03:00"
            }]
        }

        """
        query = {
            "task_ids": taskIds,
            "error_ids": errorIds,
            "error_time__lt": errorTimeLt,
            "error_time__gte": errorTimeGte,
            "task_type": taskType,
            "error_code": errorCode,
            "page": page,
            "page_size": pageSize,
        }
        return self.makeRequest(f"{self.baseUri}/tasks/errors", "GET", queryParams=query, **kwargs)

    def getErrorsCount(
        self,
        errorIds: Optional[List[int]] = None,
        taskIds: Optional[List[int]] = None,
        errorTimeLt: Optional[str] = None,
        errorTimeGte: Optional[str] = None,
        taskType: Optional[int] = None,
        errorCode: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get errors count with filters.

        Args:
            errorIds: error ids
            taskIds: task ids
            errorTimeLt: error time upper excluding bound
            errorTimeGte: error time lower including bound
            taskType: task type filter
            errorCode: task status filter

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> reply = client.getErrorsCount()
        >>> print(reply.statusCode)
        200
        >>> print(reply.text)
        {"count":8}

        """
        query = {
            "task_ids": taskIds,
            "error_ids": errorIds,
            "error_time__lt": errorTimeLt,
            "error_time__gte": errorTimeGte,
            "task_type": taskType,
            "error_code": errorCode,
        }
        return self.makeRequest(f"{self.baseUri}/tasks/errors/count", "GET", queryParams=query, **kwargs)

    def getError(self, errorId: int, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get error by id.

        Args:
            errorId: error id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> reply = client.getError(5808750)
        >>> print(reply.statusCode)
        200
        >>> print(reply.text)
        {
            "error_id":5808750,
            "task_id":14980,
            "subtask_id":69825,
            "error_code":28010,
            "description":"Objects not found",
            "detail":"Objects for clustering not found (empty set)",
            "additional_info":null,
            "error_time":"2019-04-29T10:53:42.417396+03:00"
        }

        """
        return self.makeRequest(f"{self.baseUri}/tasks/errors/{errorId}", "GET", **kwargs)

    def createLinkerTask(
        self,
        listId: Optional[str] = None,
        createList: Optional[int] = None,
        listUserData: Optional[str] = None,
        faceFilters: Optional[FaceFilters] = None,
        eventFilters: Optional[EventFilters] = None,
        description: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create and start linker task

        Args:
            description: description
            listId: list id to link faces to | WARNING required one of: listId or createList(1)
            createList: if 1 create list to link faces to or 0 to link them to list with id equals listId
                                                                    | WARNING required one of: listId or createList(1)
            listUserData: user data of list to be created when 'createList' is set to 1
            faceFilters: filters for geting faces from faces | WARNING: required one of: face or event Filters
            eventFilters: filters for getting faces from events | WARNING: required one of: face or event Filters

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> from datetime import datetime, timedelta
        >>> reply = client.createLinkerTask(createList=1, faceFilters=FaceFilters(createTimeGte=(datetime.now() -
        ...                                                                       timedelta(hours=1)).isoformat()),
        ...                                 listUserData="my awesome list")
        >>> print(reply.statusCode)
        202
        >>> print(reply.text)
        {"task_id":14981}

        """
        taskContent = {"content": {}}
        if description is not None:
            taskContent["description"] = description
        if listId is not None:
            taskContent["content"]["list_id"] = listId
        if createList is not None:
            taskContent["content"]["create_list"] = createList
        if listUserData is not None:
            taskContent["content"]["user_data"] = listUserData
        if faceFilters is not None:
            taskContent["content"]["filters"] = faceFilters.values
            taskContent["content"]["objects_type"] = faceFilters.objectsType
        if eventFilters is not None:
            taskContent["content"]["filters"] = eventFilters.values
            taskContent["content"]["objects_type"] = eventFilters.objectsType

        payload = RequestPayload.buildJson(body=taskContent)
        return self.makeRequest(f"{self.baseUri}/tasks/linker", "POST", body=payload, **kwargs)

    def createClusteringTask(
        self,
        threshold: Optional[float] = None,
        faceFilters: Optional[FaceFilters] = None,
        eventFilters: Optional[EventFilters] = None,
        description: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create and start clustering task

        Args:
            description: description
            threshold: threshold
            faceFilters: filters for geting faces from faces | WARNING: required one of: face or event Filters
            eventFilters: filters for getting faces from events | WARNING: required one of: face or event Filters

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> from datetime import datetime, timedelta
        >>> reply = client.createClusteringTask(faceFilters=FaceFilters(createTimeGte=(datetime.now() -
        >>>                                                             timedelta(hours=1)).isoformat()))
        >>> print(reply.statusCode)
        202
        >>> print(reply.text)
        {"task_id":14982}

        """
        taskContent = {"content": {}}
        if description is not None:
            taskContent["description"] = description
        if threshold is not None:
            taskContent["content"]["threshold"] = threshold
        if faceFilters is not None:
            taskContent["content"]["filters"] = faceFilters.values
            taskContent["content"]["objects_type"] = faceFilters.objectsType
        if eventFilters is not None:
            taskContent["content"]["filters"] = eventFilters.values
            taskContent["content"]["objects_type"] = eventFilters.objectsType

        payload = RequestPayload.buildJson(body=taskContent)
        return self.makeRequest(f"{self.baseUri}/tasks/clustering", "POST", body=payload, **kwargs)

    def createGcTask(
        self,
        target: Optional[str] = "attributes",
        createTimeLt: Optional[str] = None,
        description: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create and start gc task

        Args:
            target: garbage collection target, "attributes" to remove unattached attributes
            description: description
            createTimeLt: attributes create time upper excluding bound

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> from datetime import datetime, timedelta
        >>> reply = client.createGcTask(createTimeLt=(datetime.now() - timedelta(hours=1)).isoformat())
        >>> print(reply.statusCode)
        202
        >>> print(reply.text)
        {"task_id":14983}

        """
        taskContent = {"content": {"target": target}}
        if description is not None:
            taskContent["description"] = description
        if createTimeLt is not None:
            taskContent["content"].setdefault("filters", {})["create_time__lt"] = createTimeLt

        payload = RequestPayload.buildJson(body=taskContent)
        return self.makeRequest(f"{self.baseUri}/tasks/gc", "POST", body=payload, **kwargs)

    def createReportTask(
        self,
        taskId: int,
        csvDelimiter: str = ",",
        columns: Optional[List[str]] = None,
        description: Optional[str] = None,
        saveImages: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create and start clustering task

        Args:
            description: description
            csvDelimiter: csv delimiter
            taskId: source task id
            columns: columns in csv report.
                    For faces ["user_data", "create_time", "lists", "external_id", "attribute_id",
                              "avatar", "event_id"]
                    For Events: ["create_time", "external_id", "handler_id", "attribute_id", "source",
                       "top_similar_face_id", "top_similar_face_list", "top_similar_face_similarity",
                       "match_result", "extract_result", "face_id", "attach_result", "gender", "age", "emotion",
                        "ethnic_group", "tags", "user_data"]
            saveImages: whether to save images in a report

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> reply = client.createReportTask(14980)
        >>> print(reply.statusCode)
        202
        >>> print(reply.text)
        {"task_id":14984}

        """
        taskContent = {"content": {"filters": {"task_id": taskId}, "csv_delimiter": csvDelimiter}}
        if description is not None:
            taskContent["description"] = description
        if columns is not None:
            taskContent["content"]["columns"] = columns
        if saveImages is not None:
            taskContent["content"]["save_images"] = saveImages

        payload = RequestPayload.buildJson(body=taskContent)
        return self.makeRequest(f"{self.baseUri}/tasks/reporter", "POST", body=payload, **kwargs)

    def getTaskResult(self, taskId: int, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get task result by task id. Optional you can set, that task must belong to account.

        Args:
            taskId: task id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` with task result will be returned.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> reply = client.getTaskResult(14982)
        >>> print(reply.statusCode)
        200
        >>> print(reply.text)
        {
            "result": {
                "clusters": [[
                    "557d54ec-29ad-4f3c-93b4-c9092ef12515"
                ]],
                "total_objects": 0,
                "total_clusters": 0
            },
            "errors": [{
                "error_id": 10,
                "task_id": 123,
                "subtask_id": 5,
                "error_code": 0,
                "description": "Faces not found",
                "detail": "One or more faces not found, including face with id '8f4f0070-c464-460b-bf78-fac225df72e9'",
                "additional_info": "8f4f0070-c464-460b-bf78-fac225df72e9",
                "error_time": "2018-08-11T09:11:41.674Z"
            }]
        }
        """
        return self.makeRequest(f"{self.baseUri}/tasks/{taskId}/result", "GET", **kwargs)

    def cancelTask(self, taskId: int, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Cancel a task by the task id. Optional you can set, that task must belong to account.

        Args:
            taskId: task id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` with task result will be returned.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> reply = client.cancelTask(14985)
        >>> print(reply.statusCode)
        202
        >>> print(reply.text)

        """
        return self.makeRequest(f"{self.baseUri}/tasks/{taskId}", "PATCH", **kwargs)

    def deleteTask(self, taskId: int, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete a task by the task id. Optional you can set, that task must belong to account.

        Args:
            taskId: task id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` with task result will be returned.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> reply = client.deleteTask(14985)
        >>> print(reply.statusCode)
        204
        >>> print(reply.text)

        """
        return self.makeRequest(f"{self.baseUri}/tasks/{taskId}", "DELETE", **kwargs)

    def createCrossMatchTask(
        self,
        referenceFilters: Union[FaceFilters, EventFilters],
        candidateFilters: Union[FaceFilters, EventFilters],
        threshold: Optional[float] = None,
        limit: Optional[int] = None,
        description: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create and start clustering task

        Args:
            description: description
            threshold: threshold
            limit: maximum candidates count per reference
            referenceFilters: filters for getting references from either faces or events
            candidateFilters: filters for getting candidates from either faces or events

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with response.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        ... reply = client.createCrossMatchTask(referenceFilters=FaceFilters(createTimeGte='2029.07.13T12:30:00'),
        ...     candidateFilters=EventFilters(createTimeGte='2029.07.13T12:30:00'))
        >>> print(reply.statusCode)
        202
        >>> print(reply.text)
        {"task_id":14984}

        """
        taskContent = {
            "content": {
                "filters": {
                    "reference_type": referenceFilters.objectsType,
                    "reference_filters": referenceFilters.values,
                    "candidate_type": candidateFilters.objectsType,
                    "candidate_filters": candidateFilters.values,
                }
            }
        }
        if description is not None:
            taskContent["description"] = description
        if threshold is not None:
            taskContent["content"]["threshold"] = threshold
        if limit is not None:
            taskContent["content"]["limit"] = limit

        payload = RequestPayload.buildJson(body=taskContent)
        return self.makeRequest(f"{self.baseUri}/tasks/cross_match", "POST", body=payload, **kwargs)

    def createROCTask(
        self,
        markup: List[dict],
        accountIdFilter: Optional[str] = None,
        thresholdHitTop: Optional[float] = None,
        limit: Optional[int] = None,
        keyFPRs: Optional[List[float]] = None,
        description: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create and start ROC task

        Args:
            markup: markup list of dicts with face id and label (group id)
            accountIdFilter: account id filter
            thresholdHitTop: similarity threshold to calculate hit into the top-N
            limit: maximum candidates count per reference
            keyFPRs: roc-curve key FRP value list
            description: description

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with task id.

        >>> from luna3.backport4.httpclient import LunaHttpClient
        >>>
        >>> client = LunaHttpClient(accountId='02bbe8fe-7dd2-46bf-85c9-494693a9d30b')
        >>> reply = client.createROCTask(markup=[{"face_id": '8f4f0070-c464-460b-bf78-fac225df72e9', "label": 0}])
        >>> print(reply.statusCode)
        202
        >>> print(reply.text)
        {"task_id":14984}

        """
        taskContent = {"description": description, "content": {"markup": markup, "filters": {}}}

        if accountIdFilter is not None:
            taskContent["content"]["filters"]["account_id"] = accountIdFilter
        if thresholdHitTop is not None:
            taskContent["content"]["threshold_hit_top"] = thresholdHitTop
        if limit is not None:
            taskContent["content"]["limit"] = limit
        if keyFPRs is not None:
            taskContent["content"]["key_FPRs"] = keyFPRs

        payload = RequestPayload.buildJson(body=taskContent)
        return self.makeRequest(f"{self.baseUri}/tasks/roc", "POST", body=payload, **kwargs)
