"""Timers

Timer help utilities

Attributes:
    DEFAULT_LOGGER (Logger): logger for objects without own logger.
"""

import asyncio
import logging
import sys
import time
from contextlib import contextmanager
from functools import wraps
from typing import Callable, Optional


def _createLogger() -> logging.Logger:
    """Create default logger"""
    logger = logging.getLogger("timer")
    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(logging.DEBUG)
    logger.setLevel(logging.DEBUG)
    logger.addHandler(handler)
    return logger


# setup default logger configuration
# logging.basicConfig()
# logging.getLogger('timer').setLevel(logging.DEBUG)
DEFAULT_LOGGER = _createLogger()


def setDefaultLogger(logger: Optional[logging.Logger] = None):
    """
    Change global module logger

    Args:
        logger: logger
    """
    global DEFAULT_LOGGER  # pylint: disable=W0603
    if logger:
        DEFAULT_LOGGER = logger


def timer(extLogger: Optional[logging.Logger] = None) -> Callable:
    """
    Decorator for function work time estimation.

    Args:
        extLogger: external logger
    Returns:
        decorated function.
    """
    global DEFAULT_LOGGER  # pylint: disable=global-variable-not-assigned
    extLogger = extLogger or DEFAULT_LOGGER

    def timerWrapper(func: callable) -> Callable:
        if asyncio.iscoroutinefunction(func):

            @wraps(func)
            async def wrap(*func_args, **func_kwargs):
                start = time.time()
                res = await func(*func_args, **func_kwargs)
                end = time.time()
                _objectPrintLogger(func_args[0] if func_args else None, func, end - start, extLogger)
                return res

        else:

            @wraps(func)
            def wrap(*func_args, **func_kwargs):
                start = time.time()
                res = func(*func_args, **func_kwargs)
                end = time.time()
                _objectPrintLogger(func_args[0] if func_args else None, func, end - start, extLogger)
                return res

        return wrap

    return timerWrapper


@contextmanager
def contextTime(message: str, logger: logging.Logger = None):
    """
    Context manager for logging context execution time

    Args:
        message: log message
        logger: logger, by default use defaultLogger

    Yields:
        checkpoint function for logging duration between start or last checkpoint
    """
    global DEFAULT_LOGGER  # pylint: disable=global-variable-not-assigned
    logger = logger or DEFAULT_LOGGER

    def checkpoint(checkpointMessage: str = message):
        nonlocal lastCheckpoint
        _simplePrintLogger(checkpointMessage, time.time() - lastCheckpoint, logger)
        lastCheckpoint = time.time()

    _simplePrintLogger("Begin block {}".format(message), 0, logger)
    start = time.time()
    lastCheckpoint = start

    yield checkpoint

    _simplePrintLogger("End block {}".format(message), time.time() - start, logger)


def _objectPrintLogger(self: Optional[object], func: callable, duration: float, extLogger: logging.Logger) -> None:
    """
    Print timing to log

    Args:
        self: class instance with logger
        func: measured function
        duration: calling duration of function in secs
        extLogger: external logger
    """
    if hasattr(self, "logger"):
        self.logger.debug("{} time: {}".format(func.__qualname__, round(duration, 5)))
    else:
        extLogger.debug("{} time: {}".format(func.__qualname__, round(duration, 5)))


def _simplePrintLogger(message: str, duration: float, logger: logging.Logger) -> None:
    """
    Print timing to log

    Args:
        message: print message
        duration: calling duration of function in secs
    """
    logger.debug("{} time: {}".format(message, round(duration, 5)))
