"""
Module realize lists mixin
"""

from typing import Awaitable, List, Optional, Union

from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.lunavl.mixins.base import MixinStub


class ListsMixin(MixinStub):
    """Lists Mixin to work with lists."""

    def createList(self, userData: Optional[str] = None, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create new list

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/createList

        Args:
            userData: user data associated with the list (<= 128 characters)

        Returns:
            LunaResponse: body contains json with information about created list.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> response = client.createList("strange_persons")
        >>> print(response.statusCode)
        201
        >>> print(json.dumps(response.json, sort_keys=True))  # doctest: +ELLIPSIS
        {"external_url": "http://127.0.0.1:5000/6/lists/...", "list_id":..., "url":...}

        """
        payload = RequestPayload.buildJson(body={"user_data": userData}) if userData is not None else None
        return self.makeRequest(f"{self.baseUri}/lists", "POST", body=payload, **kwargs)

    def getLists(
        self,
        createTimeLt: Optional[str] = None,
        createTimeGte: Optional[str] = None,
        lastUpdateTimeLt: Optional[str] = None,
        lastUpdateTimeGte: Optional[str] = None,
        userData: Optional[str] = None,
        userDataEq: Optional[str] = None,
        listIds: Optional[List[str]] = None,
        listIdLt: Optional[str] = None,
        listIdGte: Optional[str] = None,
        page: int = 1,
        pageSize: int = 10,
        accountIdFilter: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get lists by filters

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/getLists

        Args:
            userData: Find all objects with user_data which is "like" this parameter
            userDataEq: user data, case sensitive
            listIds: list ids
            listIdLt: upper bound of list id
            listIdGte: lower bound of list id
            createTimeLt: upper bound of list create time
            createTimeGte: lower bound of list create time
            lastUpdateTimeLt: upper bound of list last update time
            lastUpdateTimeGte: lower bound of list last update time
            page: The number of page (>= 1)
            pageSize: The numbers of items on page [ 1 .. 100 ]
            accountIdFilter: account id to get lists

        Returns:
            LunaResponse: body contains json with lists that were found

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> listId = client.createList("strange_persons", raiseError=True).json["list_id"]
        >>> response = client.getLists(listIds=[listId])
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4)) # doctest: +ELLIPSIS
        {
            "lists": [
                {
                    "account_id": ...,
                    "create_time": ...,
                    "last_update_time": ...,
                    "list_id": ...,
                    "user_data": "strange_persons"
                }
            ]
        }

        """
        queries = {
            "page": page,
            "page_size": pageSize,
            "user_data": userData,
            "create_time__lt": createTimeLt,
            "create_time__gte": createTimeGte,
            "last_update_time__lt": lastUpdateTimeLt,
            "last_update_time__gte": lastUpdateTimeGte,
            "user_data__eq": userDataEq,
            "list_ids": listIds,
            "list_id__lt": listIdLt,
            "list_id__gte": listIdGte,
            "account_id": accountIdFilter,
        }
        return self.makeRequest(f"{self.baseUri}/lists", "GET", queryParams=queries, **kwargs)

    def deleteLists(
        self, listIds: List[str], withFaces: int = 0, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete lists. Optional you can set, that lists must belong to account.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/deleteLists

        Args:
            listIds: list ids
            withFaces: remove lists and all the faces contained in the lists

        Returns:
            LunaResponse: not contains response body.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> listId = client.createList("strange_persons", raiseError=True).json["list_id"]
        >>> response = client.deleteLists(listIds=[listId])
        >>> print(response.statusCode)
        204
        >>> listId = client.createList("happy_persons", raiseError=True).json["list_id"]
        >>> response = client.deleteLists(listIds=[listId], withFaces = 1)
        >>> print(response.statusCode)
        202

        """
        queries = {"with_faces": withFaces}
        payload = RequestPayload.buildJson(body={"list_ids": listIds})
        return self.makeRequest(f"{self.baseUri}/lists", "DELETE", queryParams=queries, body=payload, **kwargs)

    def getListsCount(
        self,
        listIds: Optional[List[str]] = None,
        listIdLt: Optional[str] = None,
        listIdGte: Optional[str] = None,
        createTimeLt: Optional[str] = None,
        createTimeGte: Optional[str] = None,
        lastUpdateTimeLt: Optional[str] = None,
        lastUpdateTimeGte: Optional[str] = None,
        userData: Optional[str] = None,
        userDataEq: Optional[str] = None,
        accountIdFilter: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get lists count by filters

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/getListsCount

        Args:
            userData: Find all objects with user_data which is "like" this parameter
            listIds: list ids
            listIdLt: upper bound of list id
            listIdGte: lower bound of list id
            userDataEq: user data
            createTimeLt: upper bound of list create time
            createTimeGte: lower bound of list create time
            lastUpdateTimeLt: upper bound of list last update time
            lastUpdateTimeGte: lower bound of list last update time
            accountIdFilter: account id to count lists

        Returns:
            LunaResponse: body contains json with count of lists that were found

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> listId = client.createList("strange_persons", raiseError=True).json["list_id"]
        >>> response = client.getListsCount(createTimeGte="2022-03-21T09:11:41.674Z")
        >>> print(response.statusCode)
        200
        >>> print(response.json)  # doctest: +ELLIPSIS
        {'lists_count': ...}

        """
        queries = {
            "user_data": userData,
            "list_ids": listIds,
            "list_id__lt": listIdLt,
            "list_id__gte": listIdGte,
            "create_time__lt": createTimeLt,
            "create_time__gte": createTimeGte,
            "last_update_time__lt": lastUpdateTimeLt,
            "last_update_time__gte": lastUpdateTimeGte,
            "user_data__eq": userDataEq,
            "account_id": accountIdFilter,
        }
        return self.makeRequest(f"{self.baseUri}/lists/count", "GET", queryParams=queries, **kwargs)

    def getList(
        self, listId: str, accountIdFilter: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get list by id

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/getList

        Args:
            listId: list id
            accountIdFilter: account id to get list

        Returns:
            LunaResponse: body contains json with info about list.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> listId = client.createList("strange_persons", raiseError=True).json["list_id"]
        >>> response = client.getList(listId=listId)
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4)) # doctest: +ELLIPSIS
        {
            "account_id": ...,
            "create_time": ...,
            "last_update_time": ...,
            "list_id": ...,
            "user_data": "strange_persons"
        }

        """
        queries = {"account_id": accountIdFilter}
        return self.makeRequest(f"{self.baseUri}/lists/{listId}", "GET", queryParams=queries, **kwargs)

    def checkList(
        self, listId: str, accountIdFilter: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check list existence

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/checkList

        Args:
            listId: list id
            accountIdFilter: account id to check face

        Returns:
            LunaResponse: not contains response body.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> listId = client.createList("strange_persons", raiseError=True).json["list_id"]
        >>> response = client.checkList(listId=listId)
        >>> print(response.statusCode)
        200

        """
        queries = {"account_id": accountIdFilter}
        return self.makeRequest(f"{self.baseUri}/lists/{listId}", "HEAD", queryParams=queries, **kwargs)

    def updateList(self, listId: str, userData: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Update the list user_data

        Args:
            listId: list id
            userData: user data associated with the list (<= 128 characters)

        Returns:
            LunaResponse: not contains response body.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> listId = client.createList("strange_persons", raiseError=True).json["list_id"]
        >>> response = client.updateList(listId=listId, userData="test_data12_updated")
        >>> print(response.statusCode)
        204

        """
        payload = RequestPayload.buildJson(body={"user_data": userData})
        return self.makeRequest(f"{self.baseUri}/lists/{listId}", "PATCH", body=payload, **kwargs)

    def deleteList(self, listId: str, withFaces: int = 0, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Remove list by id

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/deleteList

        Args:
            listId: list id
            withFaces: remove lists and all the faces contained in the lists

        Returns:
            LunaResponse: not contains response body.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> listId = client.createList("strange_persons", raiseError=True).json["list_id"]
        >>> response = client.deleteList(listId=listId)
        >>> print(response.statusCode)
        204
        >>> listId = client.createList("happy_persons", raiseError=True).json["list_id"]
        >>> response = client.deleteList(listId=listId, withFaces = 1)
        >>> print(response.statusCode)
        202

        """
        queries = {"with_faces": withFaces}
        return self.makeRequest(f"{self.baseUri}/lists/{listId}", "DELETE", queryParams=queries, **kwargs)

    def linkFacesToList(
        self, listId: str, action: str, faceIds: list, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        attach/detach faces to the list

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/linkFaces

        Args:
            listId: list id
            action: expected action with list and faces (Enum:"attach" "detach")
            faceIds: faces for attach or detach to/from list

        Raises:
            ValueError: if action parameter not in ("attach", "detach")

        Returns:
            LunaResponse: not contains response body.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>>
        >>> faceId = client.createFace(raiseError=True).json["face_id"]
        >>> listId = client.createList("happy_persons", raiseError=True).json["list_id"]
        >>> response = client.linkFacesToList(listId=listId, action='attach', faceIds=[faceId])
        >>> print(response.statusCode)
        204
        >>> response = client.linkFacesToList(listId=listId, action='detach', faceIds=[faceId])
        >>> print(response.statusCode)
        204

        """
        if action not in ("attach", "detach"):
            raise ValueError("Attach parameter should be equal to 'attach' or 'detach'")
        payload = RequestPayload.buildJson(body={"action": action, "face_ids": faceIds})
        return self.makeRequest(f"{self.baseUri}/lists/{listId}/faces", "PATCH", body=payload, **kwargs)
